@@ -195,8 +195,8 @@ test_BAM_removeEdge <- function()
     ## removing nothing does nothing
     c0 <- character(0)
     checkEquals(edges(g1), edges(removeEdge(c0, c0, g1)))
-    ## there is no y => a edge
-    checkEquals(edges(g1), edges(removeEdge("y", "a", g1)))
+    ## there is no y => a edge, throw error
+    checkException(removeEdge("y", "a", g1))
 
     g2 <- removeEdge("c", "a", g1)
     checkEquals(list(c=character(0)), edges(g2, "c"))
@@ -618,15 +618,14 @@ test_BAM_Union_UnDirected <- function() {
     weight=c(3.2, 1.2, 2.1, 3.2, 3.5)
     df <- data.frame(from, to, weight)
     g2 <- graphBAM(df, nodes = c("a","b","c", "d", "x", "y", "z"), 
-            edgemode = "undirected")
-
+        edgemode = "undirected")
     g <- graphUnion(g1,g2)
     checkEquals(union(nodes(g1), nodes(g2)), nodes(g))
     checkEquals(FALSE, isDirected(g))
     df <- extractFromTo(g)
     tmp <- data.frame(from = c("a", "b", "b", "c", "d", "d"),
-                      to = c("b", "c", "d", "d", "x", "y"),
-                      weight = c( NA, NA, 2.1, 3.2, 3.5, 5.4))
+        to = c("b", "c", "d", "d", "x", "y"),
+        weight = c( NA, NA, 2.1, 3.2, 3.5, 5.4))
     checkEquals(tmp, df)
 }
 
@@ -709,8 +708,8 @@ test_BAM_directed_attrs <- function() {
     bam <- graphBAM(df, edgemode = "directed")
     
     checkException(edgeData(bam,from="a", attr="code"))
-    edgeData(bam, attr = "weight") <- 1
-    edgeData(bam, attr = "code") <- "plain"
+    edgeDataDefaults(bam, attr ="weight") <- 1
+    edgeDataDefaults(bam, attr = "code") <- "plain"
 
     res <- unlist(edgeData(bam,from="a", attr="code"))
     nmres <- paste(c("a","a","a"), c ("b", "c", "x"), sep="|")  
@@ -737,8 +736,9 @@ test_BAM_undirected_attrs <- function() {
     df <- data.frame(from, to, weight)
     bam <- graphBAM(df, edgemode = "undirected")
     checkException(edgeData(bam,from="a", attr="code"))
-    edgeData(bam, attr = "weight")  <- 1
-    edgeData(bam, attr = "code") <- "plain"
+
+    edgeDataDefaults(bam, attr = "weight") <- 1
+    edgeDataDefaults(bam, attr = "code") <- "plain"
 
     res <- unlist(edgeData(bam,from="a", attr="code"))
     nmres <- paste(c("a","a","a"), c ("b", "c", "x"), sep="|")  
@@ -760,188 +760,206 @@ test_BAM_undirected_attrs <- function() {
 
 test_graphBAM_detailed_Attribute_Intersection <- function() {
 
-## nodes a b c d x y
-from = c("a", "b", "d", "d")
-to   = c("b", "c", "y", "x")
-weight=c(1.2, 2.4, 5.4, 3.2)
-df <- data.frame(from, to, weight)
-g1 <- graphBAM(df, edgemode = "directed")
-edgeData(g1, from = from, to = to ,attr = "weight")  <- c(1.2, 2.4, 5.4, 3.2)
+    ## nodes a b c d x y
+    from = c("a", "b", "d", "d")
+    to   = c("b", "c", "y", "x")
+    weight=c(1.2, 2.4, 5.4, 3.2)
+    df <- data.frame(from, to, weight)
+    g1 <- graphBAM(df, edgemode = "directed")
+    edgeData(g1, from = from, to = to ,attr = "weight")  <- c(1.2, 2.4, 5.4, 3.2)
 
-edgeData(g1, from = from, to = to ,attr = "color") <-  c("red", "blue", NA, "green")
-edgeData(g1, from = from, to = to , attr = "type") <-  c("high", "low", "high", NA)
-## nodes a b c d x y z
-from = c("a", "b", "b", "d", "d")
-to   = c("b", "c", "d", "c", "x")
-weight=c(1.2, 4.2, 5.6, 2.1, 3.2)
-df <- data.frame(from, to, weight)
-g2 <- graphBAM(df, nodes = c("a","b","c", "d", "x", "y", "z"),
+
+    edgeDataDefaults(g1, attr = "color") <- "unknown"
+    edgeDataDefaults(g1, attr ="type") <- "unknown"
+    edgeData(g1, from = from, to = to ,attr = "color") <-  c("red", "blue", NA, "green")
+    edgeData(g1, from = from, to = to , attr = "type") <-  c("high", "low", "high", NA)
+    ## nodes a b c d x y z
+    from = c("a", "b", "b", "d", "d")
+    to   = c("b", "c", "d", "c", "x")
+    weight=c(1.2, 4.2, 5.6, 2.1, 3.2)
+    df <- data.frame(from, to, weight)
+    g2 <- graphBAM(df, nodes = c("a","b","c", "d", "x", "y", "z"),
         edgemode = "directed")
-edgeData(g2, from = from, to = to,  attr = "color") <- c("red", "blue", NA, "red",
-                                                         "yellow")
-g <- graphIntersect(g1, g2)
-df <- extractFromTo(g)
-tmp <- data.frame( from = c("a", "b", "d"), 
-                     to = c("b", "c", "x"), 
-                 weight = c(1.2, NA, 3.2))
+    edgeDataDefaults(g2, attr = "color") <- "unknown"
+    edgeData(g2, from = from, to = to,  attr = "color") <- c("red", "blue", NA, "red",
+        "yellow")
+    g <- graphIntersect(g1, g2)
+    df <- extractFromTo(g)
+    tmp <- data.frame( from = c("a", "b", "d"), 
+        to = c("b", "c", "x"), 
+        weight = c(1.2, NA, 3.2))
 
-checkEquals(tmp, df)
+    checkEquals(tmp, df)
 
-attColor <- edgeData(g, attr = "color")
-nms <- paste(c("a", "b", "d"),  c("b", "c", "x"), sep = "|")
-target <- structure( c("red", "blue", NA), names = nms)
-checkEquals(target, unlist(attColor))
+    attColor <- edgeData(g, attr = "color")
+    nms <- paste(c("a", "b", "d"),  c("b", "c", "x"), sep = "|")
+    target <- structure( c("red", "blue", NA), names = nms)
+    checkEquals(target, unlist(attColor))
 
-checkException(edgeData(g, attr = "type"))
+    checkException(edgeData(g, attr = "type"))
 
-weightFun <- function(x, y) {
-   return(x +y )
-}
+    weightFun <- function(x, y) {
+        return(x +y )
+    }
 
-colorFun <- function(x,y) {
-    if(x=="red" && y == "red")
-        return("white")
-    else
-        return("black")
-}
-
-setClass("myType", representation = representation(typ ="character")) 
-myType <- function(typ){ new("myType", typ = typ)}
-typeFun <- function(x,y) {
-    if(is(x, "myType")  && is(y, "myType")){
-          if(x@typ =="low" || y@typ == "med")
-            return("low")
-         else
-            return("high")
+    colorFun <- function(x,y) {
+        if(x=="red" && y == "red")
+            return("white")
+        else
+            return("black")
         }
-        else {return (NA)}
-        
+
+    setClass("myType", representation = representation(typ ="character")) 
+    myType <- function(typ){ new("myType", typ = typ)}
+    typeFun <- function(x,y) {
+            if(is(x, "myType")  && is(y, "myType")){
+                if(x@typ =="low" || y@typ == "med")
+                    return("low")
+                else
+                    return("high")
+                }
+            else {return (NA)}
+    }
+    nodeDataDefaults(g1, attr ="color") <- "unknown"
+    nodeDataDefaults(g1, attr ="type") <- "unknown"
+    nodeDataDefaults(g2, attr ="color") <- "unknown"
+    nodeDataDefaults(g2, attr ="type") <- "unknown"
+
+    nodeData(g1,n = c("a", "b", "c"), attr ="color") <- c("red", "green", "blue")
+    nodeData(g1,n = c("b", "c"), attr ="type") <- c(myType("low"), myType("high"))
+    nodeData(g2,n = c("a", "b", "c"), attr ="color") <- c("red", "green", "red")
+    nodeData(g2,n = c("b", "c"), attr ="type") <- c(myType("med"), myType("low"))
+    g <- graphIntersect(g1, g2, nodeFun = list(type = typeFun),
+        edgeFun = list(weight = weightFun, color = colorFun))
+
+    attWeight <- edgeData(g, attr = "weight")
+    nms <- paste(c("a", "b", "d"),  c("b", "c", "x"), sep = "|")
+    target <- structure( c( 2.4, 6.6, 6.4), names = nms)
+    checkEquals(target, unlist(attWeight))
+
+    attColor <- edgeData(g, attr = "color")
+    nms <- paste(c("a", "b", "d"),  c("b", "c", "x"), sep = "|")
+    target <- structure( c( 2.4, 6.6, 6.4), names = nms)
+    checkEquals(target, unlist(attWeight))
+
+    nodeColor <- nodeData(g, attr = "color")
+    target <-  as.list(structure(c("red", "green", NA, "unknown", "unknown",
+                "unknown"), names = c("a", "b", "c", "d", "x", "y")))
+    checkEquals(target, nodeColor)
+
+    nodeType <- nodeData(g, attr = "type")
+    target <-  as.list(structure(c("unknown", "low", "high", "unknown",
+                "unknown", "unknown"), names = c("a", "b", "c", "d", "x", "y")))
+    checkEquals(target, nodeType)
 }
-nodeData(g1,n = c("a", "b", "c"), attr ="color") <- c("red", "green", "blue")
-nodeData(g1,n = c("b", "c"), attr ="type") <- c(myType("low"), myType("high"))
-nodeData(g2,n = c("a", "b", "c"), attr ="color") <- c("red", "green", "red")
-nodeData(g2,n = c("b", "c"), attr ="type") <- c(myType("med"), myType("low"))
-g <- graphIntersect(g1, g2, nodeFun = list(type = typeFun),
-            edgeFun = list(weight = weightFun, color = colorFun))
-
-attWeight <- edgeData(g, attr = "weight")
-nms <- paste(c("a", "b", "d"),  c("b", "c", "x"), sep = "|")
-target <- structure( c( 2.4, 6.6, 6.4), names = nms)
-checkEquals(target, unlist(attWeight))
-
-attColor <- edgeData(g, attr = "color")
-nms <- paste(c("a", "b", "d"),  c("b", "c", "x"), sep = "|")
-target <- structure( c( 2.4, 6.6, 6.4), names = nms)
-checkEquals(target, unlist(attWeight))
-
-nodeColor <- nodeData(g, attr = "color")
-target <-  as.list(structure(c("red", "green", NA, NA, NA, NA), 
-                 names = c("a", "b", "c", "d", "x", "y")))
-checkEquals(target, nodeColor)
-
-nodeType <- nodeData(g, attr = "type")
-cn <- as.character(NA)
-target <-  as.list(structure(c(cn, "low", "high", cn, cn, cn), 
-                 names = c("a", "b", "c", "d", "x", "y")))
-checkEquals(target, nodeType)
-
-}
-
 
 test_graphBAM_detailed_Attribute_Union <- function() {
 
-## nodes a b c d x y
-from = c("a", "b", "d", "d")
-to   = c("b", "c", "y", "x")
-weight=c(1.2, 2.4, 5.4, 3.2)
-df <- data.frame(from, to, weight)
-g1 <- graphBAM(df, edgemode = "directed")
-edgeData(g1, from = from, to = to ,attr = "weight")  <- c(1.2, 2.4, 5.4, 3.2)
-
-edgeData(g1, from = from, to = to ,attr = "color") <-  c("red", "blue", NA, "green")
-edgeData(g1, from = from, to = to , attr = "type") <-  c("high", "low", "high", NA)
-## nodes a b c d x y z
-from = c("a", "b", "b", "d", "d")
-to   = c("b", "c", "d", "c", "x")
-weight=c(1.2, 4.2, 5.6, 2.1, 3.2)
-df <- data.frame(from, to, weight)
-g2 <- graphBAM(df, nodes = c("a","b","c", "d", "x", "y", "z"),
+    ## nodes a b c d x y
+    from = c("a", "b", "d", "d")
+    to   = c("b", "c", "y", "x")
+    weight=c(1.2, 2.4, 5.4, 3.2)
+    df <- data.frame(from, to, weight)
+    g1 <- graphBAM(df, edgemode = "directed")
+    edgeData(g1, from = from, to = to ,attr = "weight")  <- c(1.2, 2.4, 5.4, 3.2)
+    
+    edgeDataDefaults(g1, attr = "color")  <- "cyan"
+    edgeDataDefaults(g1, attr = "type")  <- "unknown"
+    edgeData(g1, from = from, to = to ,attr = "color") <-  c("red", "blue", NA, "green")
+    edgeData(g1, from = from, to = to , attr = "type") <-  c("high", "low", "high", NA)
+  
+    ## nodes a b c d x y z
+    from = c("a", "b", "b", "d", "d")
+    to   = c("b", "c", "d", "c", "x")
+    weight=c(1.2, 4.2, 5.6, 2.1, 3.2)
+    df <- data.frame(from, to, weight)
+    g2 <- graphBAM(df, nodes = c("a","b","c", "d", "x", "y", "z"),
         edgemode = "directed")
-edgeData(g2, from = from, to = to,  attr = "color") <- c("red", "blue", NA, "red",
-                                                         "yellow")
-g <- graphUnion(g1, g2)
-df <- extractFromTo(g)
-tmp <- data.frame( from = c("a", "b", "d", "b", "d", "d"), 
-                     to = c("b", "c", "c", "d", "x", "y"), 
-                 weight = c(1.2, NA, 2.1, 5.6, 3.2, 5.4))
-checkEquals(tmp, df)
+    edgeDataDefaults(g2, attr = "color")  <- "cyan"
 
-attColor <- edgeData(g, attr = "color")
-nms <- paste(c("a", "b", "d", "b", "d", "d"),  c("b", "c", "c", "d", "x", "y"), sep = "|")
-target <- structure( c("red", "blue", "red", NA, NA, NA), names = nms)
-checkEquals(target, unlist(attColor))
+    edgeData(g2, from = from, to = to,  attr = "color") <- c("red", "blue", NA, "red",
+        "yellow")
+    g <- graphUnion(g1, g2)
+    df <- extractFromTo(g)
+    tmp <- data.frame( from = c("a", "b", "d", "b", "d", "d"), 
+        to = c("b", "c", "c", "d", "x", "y"), 
+        weight = c(1.2, NA, 2.1, 5.6, 3.2, 5.4))
+    checkEquals(tmp, df)
 
-attType <- edgeData(g, attr = "type")
-nms <- paste(c("a", "b", "d", "b", "d", "d"),  c("b", "c", "c", "d", "x", "y"), sep = "|")
-target <- structure( c("high", "low", NA, NA, NA, "high"), names = nms)
-checkEquals(target, unlist(attType))
+    attColor <- edgeData(g, attr = "color")
+    nms <- paste(c("a", "b", "d", "b", "d", "d"),  c("b", "c", "c", "d", "x", "y"), sep = "|")
+    target <- structure( c("red", "blue", "red", NA, NA, NA), names = nms)
+    checkEquals(target, unlist(attColor))
+
+    attType <- edgeData(g, attr = "type")
+    nms <- paste(c("a", "b", "d", "b", "d", "d"),  c("b", "c", "c", "d", "x", "y"), sep = "|")
+    target <- structure( c("high", "low", NA, NA, NA, "high"), names = nms)
+    checkEquals(target, unlist(attType))
 
 
-weightFun <- function(x, y) {
-   return(x + y )
-}
+    weightFun <- function(x, y) {
+        return(x + y )
+    }
 
-colorFun <- function(x,y) {
-    if(x=="red" || y == "red")
-        return("white")
-    else
-        return("black")
-}
+    colorFun <- function(x,y) {
+        if(x=="red" || y == "red")
+            return("white")
+        else
+            return("black")
+    }
 
-setClass("myType", representation = representation(typ ="character")) 
-myType <- function(typ){ new("myType", typ = typ)}
-typeFun <- function(x,y) {
-    if(is(x, "myType")  && is(y, "myType")){
-          if(x@typ =="low" || y@typ == "med")
-            return("low")
-         else
-            return("high")
-        }
+    setClass("myType", representation = representation(typ ="character")) 
+    myType <- function(typ){ new("myType", typ = typ)}
+    typeFun <- function(x,y) {
+        if(is(x, "myType")  && is(y, "myType")){
+            if(x@typ =="low" || y@typ == "med")
+                return("low")
+            else
+                return("high")
+            }
         else {return (NA)}
-        
-}
-nodeData(g1,n = c("a", "b", "c"), attr ="color") <- c("red", "green", "blue")
-nodeData(g1,n = c("b", "c"), attr ="type") <- c(myType("low"), myType("high"))
-nodeData(g2,n = c("a", "b", "c", "z"), attr ="color") <- c("red", "green", "red","pink")
-nodeData(g2,n = c("b", "c"), attr ="type") <- c(myType("med"), myType("low"))
-nodeData(g2,n = c("a", "b", "c"), attr = "test") <- c("pass", "fail", "pass")
+    }
+   
+    nodeDataDefaults(g1, attr ="color") <- "cyan"
+    nodeDataDefaults(g1, attr="type") <- "unknown"
+    nodeData(g1,n = c("a", "b", "c"), attr ="color") <- c("red", "green", "blue")
+    nodeData(g1,n = c("b", "c"), attr ="type") <- c(myType("low"), myType("high"))
+    
+    nodeDataDefaults(g2, attr ="color") <- "cyan"
+    nodeDataDefaults(g2, attr="type") <- "unknown"
+    nodeDataDefaults(g2, attr="test") <- "missing"
+
+    nodeData(g2,n = c("a", "b", "c", "z"), attr ="color") <- c("red", "green", "red","pink")
+    nodeData(g2,n = c("b", "c"), attr ="type") <- c(myType("med"), myType("low"))
+    nodeData(g2,n = c("a", "b", "c"), attr = "test") <- c("pass", "fail", "pass")
 
 
-g <- graphUnion(g1, g2, edgeFun = list(weight = weightFun, color = colorFun))
+    g <- graphUnion(g1, g2, edgeFun = list(weight = weightFun, color = colorFun))
 
-attWeight <- edgeData(g, attr = "weight")
-nms <- paste(c("a", "b", "d", "b", "d", "d"),  c("b", "c", "c", "d", "x", "y"), sep = "|")
-target <- structure( c( 2.4, 6.6, 2.1, 5.6, 6.4, 5.4), names = nms)
-checkEquals(target, unlist(attWeight))
+    attWeight <- edgeData(g, attr = "weight")
+    nms <- paste(c("a", "b", "d", "b", "d", "d"),  c("b", "c", "c", "d", "x", "y"), sep = "|")
+    target <- structure( c( 2.4, 6.6, 2.1, 5.6, 6.4, 5.4), names = nms)
+    checkEquals(target, unlist(attWeight))
 
-attColor <- edgeData(g, attr = "color")
-nms <- paste(c("a", "b", "d", "b", "d", "d"),  c("b", "c", "c", "d", "x", "y"), sep = "|")
-target <- structure(c( "white", "black", "red", NA, "black", NA), names = nms)
-checkEquals( target, unlist(attColor))
+    attColor <- edgeData(g, attr = "color")
+    nms <- paste(c("a", "b", "d", "b", "d", "d"),  c("b", "c", "c", "d", "x", "y"), sep = "|")
+    target <- structure(c( "white", "black", "red", NA, "black", NA), names = nms)
+    checkEquals( target, unlist(attColor))
 
-attType <- edgeData(g, attr = "type")
-nms <- paste(c("a", "b", "d", "b", "d", "d"),  c("b", "c", "c", "d", "x", "y"), sep = "|")
-target <- structure( c("high", "low", NA, NA, NA, "high"), names = nms)
-checkEquals(target, unlist(attType))
+    attType <- edgeData(g, attr = "type")
+    nms <- paste(c("a", "b", "d", "b", "d", "d"),  c("b", "c", "c", "d", "x", "y"), sep = "|")
+    target <- structure( c("high", "low", NA, NA, NA, "high"), names = nms)
+    checkEquals(target, unlist(attType))
 
 
-attType <- edgeData(g, attr = "type")
-nms <- paste(c("a", "b", "d", "b", "d", "d"),  c("b", "c", "c", "d", "x", "y"), sep = "|")
-target <- structure( c("high", "low", NA, NA, NA, "high"), names = nms)
-checkEquals(target, unlist(attType))
+    attType <- edgeData(g, attr = "type")
+    nms <- paste(c("a", "b", "d", "b", "d", "d"),  c("b", "c", "c", "d", "x", "y"), sep = "|")
+    target <- structure( c("high", "low", NA, NA, NA, "high"), names = nms)
+    checkEquals(target, unlist(attType))
 
 }
 
+
 test_graphBAM_removeEdgesByWeight <- function() {
     from = c("a", "b", "d", "d")
     to   = c("b", "c", "y", "x")
@@ -949,14 +967,15 @@ test_graphBAM_removeEdgesByWeight <- function() {
     df <- data.frame(from, to, weight)
     g <- graphBAM(df, edgemode = "directed")
 
+    edgeDataDefaults(g, attr="color") <- "pink"
     edgeData(g, from = from, to = to ,attr = "color") <-  c("red", "blue", NA, "green")
-  
+
     res <- removeEdgesByWeight(g, lessThan = 2.0)
     checkEquals(attr(res@edgeSet@bit_vector, "nbitset"), 2)
     checkEquals(res@edgeSet@weights, c(2.2, 2.0))
     current <- unlist( edgeData(res, attr = "color"))
     target <- structure(c("red", "blue"), 
-               names = paste(c("a", "b"), c("b", "c"), sep = "|"))
+        names = paste(c("a", "b"), c("b", "c"), sep = "|"))
     checkEquals(target, current)
 
     res <- removeEdgesByWeight(g, greaterThan = 1.9)
@@ -964,14 +983,14 @@ test_graphBAM_removeEdgesByWeight <- function() {
     checkEquals(res@edgeSet@weights, c(0.2, 0.4))
     current <- unlist( edgeData(res, attr = "color"))
     target <- structure(c("green", NA), 
-               names = paste(c("d", "d"), c("x", "y"), sep = "|"))
+        names = paste(c("d", "d"), c("x", "y"), sep = "|"))
     checkEquals(target, current)
 
     res <- removeEdgesByWeight(g, lessThan =1.0, greaterThan = 2)
     checkEquals(res@edgeSet@weights, c(2.0))
     current <- unlist( edgeData(res, attr = "color"))
     target <- structure(c("blue"), 
-               names = paste(  "b", "c", sep = "|"))
+        names = paste(  "b", "c", sep = "|"))
     checkEquals(target, current)
 
     res <- removeEdgesByWeight(g, greaterThan = 0.1)
@@ -987,12 +1006,14 @@ test_graphBAM_nodeAttributes <- function(){
     df <- data.frame(from, to, weight)
     g <- graphBAM(df, edgemode = "directed")
 
+    nodeDataDefaults(g, attr ="color") <- "blue"
     nodeData(g, n = c("d","a"), attr = "color") <- c("red", "green")
     current <- nodeData(g, attr = "color")
-    target <- as.list(structure(  c("green", NA, NA, "red", NA, NA), 
+    target <- as.list(structure(  c("green", "blue", "blue", "red", "blue", "blue"), 
                         names = c("a", "b", "c", "d", "x", "y")))
     checkEquals(target, current)
-      
+    
+    nodeDataDefaults(g, attr="mat") <- NA 
     nodeData(g, n= c("x", "y"), attr = "mat") <- df
     current <-  nodeData(g, n= c("x", "y"), attr = "mat")
     target <- list(x = df, y = df)
@@ -1000,7 +1021,7 @@ test_graphBAM_nodeAttributes <- function(){
 
     sg <- subGraph(c("d","b"), g)
     current <- nodeData(sg, attr = "color")
-    target  <- as.list(structure(c(NA, "red"), names = c("b", "d")))
+    target  <- as.list(structure(c("blue", "red"), names = c("b", "d")))
     checkEquals(target, current)
 }
 
@@ -1013,8 +1034,8 @@ test_BAM_directed_attrs_s4 <- function() {
     df <- data.frame(from, to, weight)
     bam <- graphBAM(df, edgemode = "directed")
     
-    edgeData(bam, attr = "weight") <- 1.3
-    edgeData(bam, attr = "vals") <- df
+    edgeDataDefaults(bam, attr = "weight") <- 1.3
+    edgeDataDefaults (bam, attr = "vals") <- df
     edgeData(bam, from = "a", attr= "vals") <- "unknown"
 
     res <- edgeData(bam, attr="vals")
@@ -1022,12 +1043,14 @@ test_BAM_directed_attrs_s4 <- function() {
     target <- structure(list(df, "unknown", "unknown", df, "unknown",df), names = nmres)
     checkEquals(res, target)
 
+    edgeDataDefaults(bam, attr = "mat") <- NA
     edgeData(bam,from = "a", to = "x", attr= "mat") <- matrix(1)
     res <- edgeData(bam, from = "a", attr = "mat")
     nmres <- paste(c("a", "a", "a"), c("b", "c", "x"), sep = "|")
     target <- structure( list(NA, NA, matrix(1)), names = nmres)
     checkEquals(res, target)
 
+    edgeDataDefaults(bam, attr = "mk") <- NA
     edgeData(bam,to = "c", attr= "mk") <- matrix(1)
     res <- edgeData(bam, attr = "mk")
     nmres <- paste(c("c", "a", "a", "x", "a", "x"), c("a", "b", "c", "c", "x", "y"), sep ="|")
@@ -1042,9 +1065,12 @@ test_BAM_undirected_attrs_s4 <- function() {
     weight = c(2, 1, 3, 4)
     df <- data.frame(from, to, weight)
     bam <- graphBAM(df, edgemode = "undirected")
-    
-    edgeData(bam, attr = "weight") <- 1.3
-    edgeData(bam, attr = "vals") <- df
+   
+    edgeDataDefaults(bam, attr = "weight") <- 1.3
+    edgeDataDefaults(bam, attr = "vals") <- df
+
+    #  edgeData(bam, attr = "weight") <- 1.3
+    # edgeData(bam, attr = "vals") <- df
     edgeData(bam, from = "x", attr = "vals") <- "unknown"
 
     res <- edgeData(bam, attr="vals")
@@ -1052,13 +1078,15 @@ test_BAM_undirected_attrs_s4 <- function() {
     target <- structure(list(df, df, "unknown", "unknown", df, df, "unknown", 
                     "unknown"), names = nmres)
     checkEquals(res, target)
-   
-    edgeData(bam,from = "a", to = "x", attr= "mat") <- matrix(1)
+  
+    edgeDataDefaults(bam, attr ="mat") <- NA 
+    edgeData(bam,from = "a", to = "x", attr= "mat") <-  matrix(1)
     res <- edgeData(bam, attr = "mat")
     target <- structure(list(NA, NA, matrix(1), NA, NA, NA, matrix(1), NA), 
             names = nmres)
     checkEquals(res, target)
 
+    edgeDataDefaults(bam, attr = "mk") <- NA
     edgeData(bam,to = "c", attr= "mk") <- matrix(1)
     res <- edgeData(bam, attr = "mk")
     target <- structure( list(NA, matrix(1), NA, NA, NA, matrix(1), NA ,NA), 
@@ -1082,25 +1110,28 @@ test_graphBAM_S4_Attribute_Intersection <- function() {
     g1 <- graphBAM(df, edgemode = "directed")
     edgeData(g1, from = from, to = to ,attr = "weight")  <- c(1.2, 2.4, 5.4, 3.2)
 
-    edgeData(g1, from = from, to = to ,attr = "color") <-  
-    c(myColor("red"), myColor("blue"), NA, myColor("green"))
+    edgeDataDefaults(g1, attr="color") <- "cyan"
+    edgeDataDefaults(g1, attr="type") <- "unknown"
+    edgeData(g1, from = from, to = to ,attr = "color") <-  c(myColor("red"),
+        myColor("blue"), NA, myColor("green"))
 
-    edgeData(g1, from = from, to = to , attr = "type") <-  
-    c(myType("high"), myType("low"), myType("high"), NA)
+    edgeData(g1, from = from, to = to , attr = "type") <- c(myType("high"),
+        myType("low"), myType("high"), NA)
     ## nodes a b c d x y z
     from = c("a", "b", "b", "d", "d")
     to   = c("b", "c", "d", "c", "x")
     weight=c(1.2, 4.2, 5.6, 2.1, 3.2)
     df <- data.frame(from, to, weight)
     g2 <- graphBAM(df, nodes = c("a","b","c", "d", "x", "y", "z"),
-            edgemode = "directed")
-    edgeData(g2, from = from, to = to,  attr = "color") <- 
-    c(myColor("red"),myColor("blue"), NA, myColor("red"), myColor("yellow"))
+        edgemode = "directed")
+    edgeDataDefaults(g2, attr ="color") <- "cyan"
+    edgeData(g2, from = from, to = to,  attr = "color") <- c(myColor("red"),
+        myColor("blue"), NA, myColor("red"), myColor("yellow"))
 
     g <- graphIntersect(g1, g2)
     df <- extractFromTo(g)
     tmp <- data.frame( from = c("a", "b", "d"), to = c("b", "c", "x"), 
-            weight = c(1.2, NA, 3.2))
+        weight = c(1.2, NA, 3.2))
     checkEquals(tmp, df)
 
     attColor <- edgeData(g, attr = "color")
@@ -1130,12 +1161,12 @@ test_graphBAM_S4_Attribute_Intersection <- function() {
     nms <- paste(c("a", "b", "d"),  c("b", "c", "x"), sep = "|")
     target <- structure( c("white", "black", "black"), names = nms)
     checkEquals(target, unlist(attColor))
-
     checkException(edgeData(g, attr = "type"))
 
 }
 
 test_graphBAM_S4_Attribute_Union <- function() {
+
     setClass("myColor", representation = representation(col ="character")) 
     setClass("myType", representation = representation(typ ="character")) 
     myColor <- function(col){ new("myColor", col = col)}
@@ -1149,10 +1180,12 @@ test_graphBAM_S4_Attribute_Union <- function() {
     g1 <- graphBAM(df, edgemode = "directed")
     edgeData(g1, from = from, to = to ,attr = "weight")  <- c(1.2, 2.4, 5.4, 3.2)
 
-    edgeData(g1, from = from, to = to ,attr = "color") <-  
-    c(myColor("red"), myColor("blue"), NA, myColor("green"))
-    edgeData(g1, from = from, to = to , attr = "type") <- 
-    c(myType("high"), myType("low"), myType("high"), NA)
+    edgeDataDefaults(g1, attr = "color") <- "cyan"
+    edgeDataDefaults(g1, attr = "type") <- "missing"
+    edgeData(g1, from = from, to = to ,attr = "color") <- c(myColor("red"), 
+        myColor("blue"), NA, myColor("green"))
+    edgeData(g1, from = from, to = to , attr = "type") <- c(myType("high"), 
+        myType("low"), myType("high"), NA)
     ## nodes a b c d x y z
     from = c("a", "b", "b", "d", "d")
     to   = c("b", "c", "d", "c", "x")
@@ -1160,8 +1193,9 @@ test_graphBAM_S4_Attribute_Union <- function() {
     df <- data.frame(from, to, weight)
     g2 <- graphBAM(df, nodes = c("a","b","c", "d", "x", "y", "z"),
             edgemode = "directed")
-    edgeData(g2, from = from, to = to,  attr = "color") <- 
-    c(myColor("red"), myColor("blue"), NA, myColor("red"), myColor("yellow"))
+    edgeDataDefaults(g2, attr = "color") <- "cyan"
+    edgeData(g2, from = from, to = to,  attr = "color") <- c(myColor("red"), 
+        myColor("blue"), NA, myColor("red"), myColor("yellow"))
 
     g <- graphUnion(g1, g2)
     df <- extractFromTo(g)
@@ -1215,38 +1249,41 @@ test_graphBAM_S4_Attribute_Union <- function() {
 
 }
 
-test_graphBAM_addNode <- function(){
+test_graphBAM_addNode1 <- function(){
 
     from = c("a", "b", "d", "d")
     to   = c("b", "c", "y", "x")
     weight=c(2.2, 2.0, 0.4, 0.2)
     df <- data.frame(from, to, weight)
     g <- graphBAM(df, edgemode = "directed")
-
+    nodeDataDefaults(g, attr="color") <- "pink"
     nodeData(g, n = c("d","a"), attr = "color") <- c("red", "green")
-    nodeData(g, attr= "type" ) <- "low"
+    nodeDataDefaults(g, attr="type") <- "unknown"
     nodeData(g, n = c("a", "b", "y", "d"), attr = "type") <- c("high", "med", "high", "low") 
 
     gr <- addNode(c("q", "ss"), g)
     current <- nodeData(gr, attr = "color")
-    target <- as.list(structure( c("green", NA, NA, "red", NA, NA, NA, NA), 
+    target <- as.list(structure( c("green", "pink", "pink", "red", "pink", "pink", "pink", "pink"), 
                             names = c("a", "b", "c", "d", "q", "ss", "x", "y")))
     checkEquals(target, current)
     
     current <- nodeData(gr, attr = "type")
-    target <- as.list(structure( c("high", "med", "low", "low", NA, NA, "low", "high"), 
+    target <- as.list(structure( c("high", "med", "unknown", "low", "unknown",
+                "unknown", "unknown", "high"), 
                             names = c("a", "b", "c", "d", "q", "ss", "x", "y")))
     checkEquals(target, current)
 }
 
-test_graphBAM_addNode <- function(){
+test_graphBAM_addNode2 <- function(){
 
     from = c("a", "b", "d", "d")
     to   = c("b", "c", "y", "x")
     weight=c(2.2, 2.0, 0.4, 0.2)
     df <- data.frame(from, to, weight)
     g <- graphBAM(df, edgemode = "directed")
-
+    
+    edgeDataDefaults(g, attr="color") <- "blue"
+    edgeDataDefaults(g, attr="type") <- "unknown"
     edgeData(g, from = c("d","a"), to = c("y", "b"), attr = "color") <- c("red", "green")
     edgeData(g, from = c("a", "b"), to = c("b", "c") , attr = "type") <- c("low", "high")
     g1 <- addEdge(from = c("d", "b"), to = c("c", "x"), g, weights = c(4.0, 10.0))
@@ -1257,19 +1294,24 @@ test_graphBAM_addNode <- function(){
     checkEquals(target, current)
       
     current <- edgeData(g1, attr ="color")
-    lbl <- paste(c("a", "b", "d", "b", "d", "d"), c( "b", "c", "c", "x", "x", "y") , sep ="|")
-    target <- as.list( structure(c("green", NA, NA, NA, NA, "red"), names = lbl))
+    lbl <- paste(c("a", "b", "d", "b", "d", "d"), 
+                c( "b", "c", "c", "x", "x", "y"), sep ="|")
+    target <- as.list( structure(c("green", "blue", "blue", "blue", "blue", "red"),
+                 names = lbl))
     checkEquals(target, current)
 
     current <- edgeData(g1, attr ="type")
-    lbl <- paste(c("a", "b", "d", "b", "d", "d"), c( "b", "c", "c", "x", "x", "y") , sep ="|")
-    target <- as.list( structure(c("low", "high", NA, NA, NA, NA), names = lbl))
+    lbl <- paste(c("a", "b", "d", "b", "d", "d"), 
+                c( "b", "c", "c", "x", "x", "y") , sep ="|")
+    target <- as.list( structure(c("low", "high", "unknown", "unknown", "unknown", "unknown"),
+             names = lbl))
     checkEquals(target, current)
 }
 
 
 
 test_graphBAM_nodeUnion_Attributes <- function(use.factors=TRUE){
+   
     setClass("myType", representation = representation(typ ="character")) 
     myType <- function(typ){ new("myType", typ = typ)}
     testFun <- function(x,y) {
@@ -1280,7 +1322,6 @@ test_graphBAM_nodeUnion_Attributes <- function(use.factors=TRUE){
             else
                 return("ab")
         } else return(as.character(NA))
-
     }
     funList <- structure(list(testFun), names ="gene")
     ft1 <- data.frame(from=c("a", "a", "a", "b", "b"),
@@ -1289,6 +1330,10 @@ test_graphBAM_nodeUnion_Attributes <- function(use.factors=TRUE){
             stringsAsFactors = use.factors)
 
     g1 <- graphBAM(ft1, edgemode="directed")
+    nodeDataDefaults(g1, attr="color") <- "cyan"
+    nodeDataDefaults(g1, attr="type") <- "missing"
+    nodeDataDefaults(g1, attr="kp") <- "missing"
+    nodeDataDefaults(g1, attr="gene") <- "unknown"
     nodeData(g1, n = c("a", "b", "c") , attr = "color") <- c("red", "green", "blue")
     nodeData(g1, n = c("a", "b"), attr = "type") <- c("low", "high")
     nodeData(g1, n = c("a", "b"), attr = "kp") <- c("kplow", "kphigh")
@@ -1299,28 +1344,30 @@ test_graphBAM_nodeUnion_Attributes <- function(use.factors=TRUE){
             weight=c(6, 5, 2),
             stringsAsFactors = use.factors)
     g2 <- graphBAM(ft1,nodes = c("a","b", "c", "d", "x", "y", "z"), edgemode = "directed")
+    
+    nodeDataDefaults(g2, attr ="color") <- "cyan"
+    nodeDataDefaults(g2, attr="type") <- "missing"
+    nodeDataDefaults(g2, attr="gene") <- "unknown"
     nodeData(g2, n = c("a", "b", "x", "y", "z") , attr = "color") <- c("red", "red", "green", "pink", "yellow")
     nodeData(g2, n = c("a", "b"), attr = "type") <- c("low", "high")
     nodeData(g2, n = c("a", "b"), attr = "gene") <- c(myType("at"), myType("kt"))
 
-
     res <- graphUnion(g1, g2, nodeFun = funList)
 
     current <- nodeData(res, attr = "color")
     cn <- as.character(NA)
-    target <- as.list( structure(c("red", cn, cn, cn, "green", "pink", "yellow"), 
+    target <- as.list( structure(c("red", cn, cn, "cyan", "green", "pink", "yellow"), 
                     names = c("a", "b", "c", "d", "x", "y", "z")))
     checkEquals(target, current)
 
     current <- nodeData(res, attr = "type")
-    cn <- as.character(NA)
-    target <- as.list( structure(c("low", "high", cn, cn, cn, cn, cn), 
+    target <- as.list( structure(c("low", "high", "missing", "missing", "missing", "missing", "missing"), 
                     names = c("a", "b", "c", "d", "x", "y", "z")))
     checkEquals(target, current)
 
     current <- nodeData(res, attr = "kp")
-    cn <- as.character(NA)
-    target <- as.list( structure(c("kplow", "kphigh", cn, cn, cn, cn, cn), 
+    target <- as.list( structure(c("kplow", "kphigh", "missing", "missing", "missing",
+                "missing", "missing"), 
                     names = c("a", "b", "c", "d", "x", "y", "z")))
     checkEquals(target, current)
   
@@ -1329,7 +1376,7 @@ test_graphBAM_nodeUnion_Attributes <- function(use.factors=TRUE){
     checkEquals(target, current)
 
     current <- nodeData(res, n= c( "x", "y", "z"), attr ="gene")
-    target <- as.list( structure(c(as.logical(NA), as.logical(NA), as.logical(NA)), 
+    target <- as.list( structure(c("unknown","unknown", "unknown"), 
                     names = c("x", "y", "z")))
     checkEquals(target, current)
 }
@@ -1342,10 +1389,11 @@ test_graphBAM_removeNode <- function(){
     weight=c(2.2, 2.0, 0.4, 0.2)
     df <- data.frame(from, to, weight)
     g <- graphBAM(df, edgemode = "directed")
+    nodeDataDefaults(g, attr="name") <- "NN"
     nodeData(g, n = c("a","b", "c", "d", "x", "y"), attr = "name") <-  
              c("a", "b", "c", "d", "x", "y")
-    edgeData(g, from = from, to = to , attr = "name") <- 
-            paste(from, to , sep= "")
+    edgeDataDefaults(g, attr="name") <- "EE"
+    edgeData(g, from = from, to = to , attr = "name") <-  paste(from, to , sep= "")
 
     res <- removeNode(c("x","b"), g)
     current <- nodeData(res, attr = "name")
